import 'dart:io';

import 'package:flutter/material.dart';

class PicturesGrid extends StatelessWidget {
  final List<File>? pictures;
  final List<String>? remotePicturesUrls;
  final int height;
  final String? mainPictureFilePath;
  final String? mainRemotePictureUrl;
  final void Function(dynamic)? deleteButtonAction;
  final void Function(dynamic)? onSelectMainPicture;
  final EdgeInsetsGeometry? margin;
  final EdgeInsetsGeometry? padding;

  const PicturesGrid({
    Key? key,
    required this.pictures,
    required this.remotePicturesUrls,
    required this.height,
    this.mainPictureFilePath,
    this.mainRemotePictureUrl,
    this.onSelectMainPicture,
    this.deleteButtonAction,
    this.margin,
    this.padding,
  }) : super(key: key);

  _getImage(int index) {
    int pLen = pictures?.length ?? 0;
    int rLen = remotePicturesUrls?.length ?? 0;
    if (index >= pLen + rLen) {
      return null;
    }
    if (index >= pLen) {
      return remotePicturesUrls?[index - pLen];
    }
    return pictures?[index];
  }

  _getImageWidget(int index, int height) {
    final image = _getImage(index);
    if (image is String) {
      return Image.network(image, cacheHeight: height).image;
    }
    if (image is File) {
      return Image.file(image, cacheHeight: height).image;
    }
    throw ('Imagem não encontrada. Index $index');
  }

  _showPopupToSetMain(int index, BuildContext context, details) async {
    final _onSelectMainPicture = onSelectMainPicture;
    if (_onSelectMainPicture == null) {
      return;
    }

    final option = await showMenu(
      context: context,
      position: RelativeRect.fromLTRB(
          details.globalPosition.dx,
          details.globalPosition.dy,
          details.globalPosition.dx,
          details.globalPosition.dy),
      items: [
        PopupMenuItem(
          child: Text('Definir como imagem principal'),
          value: index,
        ),
      ],
    );
    if (option != null) {
      final image = _getImage(option);
      _onSelectMainPicture(image);
    }
  }

  _isMainPicture(int index) {
    final _mainPictureFilePath = mainPictureFilePath;
    final _mainRemotePictureUrl = mainRemotePictureUrl;
    if (_mainPictureFilePath == null && _mainRemotePictureUrl == null) {
      return false;
    }

    final _image = _getImage(index);
    if (_image is String && _image == _mainRemotePictureUrl) {
      return true;
    }
    if (_image is File && _image.path == _mainPictureFilePath) {
      return true;
    }
    return false;
  }

  @override
  Widget build(BuildContext context) {
    final _pictures = pictures;
    final _remotePicturesUrls = remotePicturesUrls;
    final _deleteButtonAction = deleteButtonAction;

    return Container(
      margin: margin,
      padding: padding,
      height: height.toDouble(),
      child: _pictures != null && _pictures.length > 0 ||
              _remotePicturesUrls != null && _remotePicturesUrls.length > 0
          ? GridView.builder(
              scrollDirection: Axis.horizontal,
              // shrinkWrap: true,
              gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
                // crossAxisCount: 3,
                maxCrossAxisExtent: 10000,
                crossAxisSpacing: 8,
                mainAxisSpacing: 8,
              ),
              itemCount:
                  (_pictures?.length ?? 0) + (_remotePicturesUrls?.length ?? 0),
              itemBuilder: (context, index) {
                return GestureDetector(
                  onLongPressStart: (details) =>
                      _showPopupToSetMain(index, context, details),
                  child: Container(
                    alignment: Alignment.bottomRight,
                    child: IconButton(
                      padding: EdgeInsets.zero,
                      color: Colors.green,
                      icon: Icon(
                        Icons.delete,
                        color: Colors.red,
                      ),
                      onPressed: _deleteButtonAction != null
                          ? () => _deleteButtonAction(_getImage(index))
                          : null,
                    ),
                    decoration: BoxDecoration(
                      border: _isMainPicture(index)
                          ? Border.all(width: 1, color: Colors.red)
                          : null,
                      // boxShadow: [BoxShadow(color: Colors.black)],
                      image: DecorationImage(
                        fit: BoxFit.cover,
                        image: _getImageWidget(index, height),
                      ),
                    ),
                  ),
                );
              })
          : SizedBox(
              height: height.toDouble(),
            ),
    );
  }
}
